@@ -79,30 +79,58 @@ FindTerm (SyntaxDef * syntax, int type, int id)
 
 
 /* debugging stuff */
-#ifdef DEBUG_PARSER
 void
-PrintFreeStorage (FreeStorageElem * storage)
+freestorage_print(char *myname, SyntaxDef *syntax, FreeStorageElem * storage, int level)
 {
-	static int    level = 0;
-
+	if( myname == NULL ) 
+		myname = MyName;
 	while (storage)
 	{
-		level++;
-		fprintf (stderr, "\n%d>Term's keyword: [%s]", level, storage->term->keyword);
-		fprintf (stderr, "\n%d>Data:", level);
+		char sub_terminator = ' ' ;
+		if( level > 0 ) 
+			if( syntax->terminator == '\n' || syntax->terminator == '\0' ) 
+				fprintf (stderr, "%*s", level*4, " ");
+
+		if( !get_flags( storage->term->flags, TF_NO_MYNAME_PREPENDING )  )
+		{	
+			fprintf (stderr, "*%s", myname);
+			if( storage->term->keyword[0] == '~' )
+				fputc( ' ', stderr );
+		}
+		fprintf (stderr, "%s", storage->term->keyword);
 		{
 			register int  i;
+			int token_count = storage->argc ; 
+			if( storage->term->sub_syntax ) 
+			{
+				token_count = GetTermUseTokensCount(storage->term);
+				if( get_flags( storage->term->flags, TF_NAMED|TF_INDEXED|TF_DIRECTION_INDEXED) )
+					++token_count;
+			}	 
 
-			for (i = 0; i < storage->argc; i++)
-				fprintf (stderr, "\n%d>  %d:\t[%s]", level, i, storage->argv[i]);
+			for (i = 0; i < token_count; i++)
+				fprintf (stderr, " %s", storage->argv[i]);
+			if( token_count == 0 && storage->sub && storage->term->sub_syntax )
+				fputc( ' ', stderr );
 		}
-		fprintf (stderr, "\n%d>Subterm: 0x%X", level, (int)(storage->sub));
-		PrintFreeStorage (storage->sub);
+		if( storage->sub && storage->term->sub_syntax ) 
+		{	
+			if( storage->term->sub_syntax->terminator == '\n' ) 
+				fputc( '\n', stderr );		
+ 			freestorage_print(myname, storage->term->sub_syntax, storage->sub, level+1);	
+			sub_terminator = storage->term->sub_syntax->file_terminator ;
+			if( sub_terminator == '\0' ) 
+				sub_terminator = '\n' ;
+		}
+		if( storage->next &&  syntax->terminator != '\0' && (sub_terminator == ' ' || sub_terminator != syntax->terminator ) ) 
+			fputc( syntax->terminator, stderr );
 		storage = storage->next;
-		level--;
 	}
+	if( syntax->file_terminator != '\0' ) 
+		fputc( syntax->file_terminator, stderr );
+	else
+		fputc( '\n', stderr );
 }
-#endif
 
 /* StringArray functionality */
 char        **
@@ -929,6 +957,54 @@ check_avail_args (FreeStorageElem * stored, int pos, int need)
 	return True;
 }
 
+int 
+direction2index(const char *ptr)	
+{
+	int index = 0;             
+    /* custom code to parse frame sides very fast :*/
+
+	if( ptr[0] == 'W' || ptr[0] == 'w' )
+        index = FR_W ;
+    else if( ptr[0] == 'E' || ptr[0] == 'e' )
+        index = FR_E ;
+    else if( ptr[0] == 'N' || ptr[0] == 'n' )
+    {
+        if( ptr[1] == 'W' || ptr[1] == 'w' )
+            index = FR_NW ;
+        else if( ptr[1] == 'E' || ptr[1] == 'e' )
+            index = FR_NE ;
+        else 
+        {
+            register int i = 0;
+            while( i < 5 && ptr[i] != '\0' ) ++i;
+            if( ptr[i] == 'W' || ptr[i] == 'w' )
+                index = FR_NW ;
+            else if( ptr[i] == 'E' || ptr[i] == 'e' )
+                index = FR_NE ;
+            else
+                index = FR_N ;
+        }
+    }else if( ptr[0] == 'S' || ptr[0] == 's' )
+    {
+        if( ptr[1] == 'W' || ptr[1] == 'w' )
+            index = FR_SW ;
+        else if( ptr[1] == 'E' || ptr[1] == 'e' )
+            index = FR_SE ;
+        else
+        {
+            register int i = 0;
+            while( i < 5 && ptr[i] != '\0' ) ++i;
+            if( ptr[i] == 'W' || ptr[i] == 'w' )
+                index = FR_SW ;
+            else if( ptr[i] == 'E' || ptr[i] == 'e' )
+                index = FR_SE ;
+            else
+                index = FR_S ;
+        }
+    }
+	return index;
+}
+
 int
 ReadConfigItem (ConfigItem * item, FreeStorageElem * stored)
 {
@@ -956,51 +1032,9 @@ ReadConfigItem (ConfigItem * item, FreeStorageElem * stored)
 			item->index = atoi (stored->argv[pos++]);
         }else if (get_flags(stored->term->flags, TF_DIRECTION_INDEXED))
         {
-            register char *ptr ;
 			if (!check_avail_args (stored, pos, 1))
 				return ret_code;
-            ptr = stored->argv[pos];
-            /* custom code to parse frame sides very fast :*/
-            if( ptr[0] == 'W' || ptr[0] == 'w' )
-                item->index = FR_W ;
-            else if( ptr[0] == 'E' || ptr[0] == 'e' )
-                item->index = FR_E ;
-            else if( ptr[0] == 'N' || ptr[0] == 'n' )
-            {
-                if( ptr[1] == 'W' || ptr[1] == 'w' )
-                    item->index = FR_NW ;
-                else if( ptr[1] == 'E' || ptr[1] == 'e' )
-                    item->index = FR_NE ;
-                else
-                {
-                    register int i = 0;
-                    while( i < 5 && ptr[i] != '\0' ) ++i;
-                    if( ptr[i] == 'W' || ptr[i] == 'w' )
-                        item->index = FR_NW ;
-                    else if( ptr[i] == 'E' || ptr[i] == 'e' )
-                        item->index = FR_NE ;
-                    else
-                        item->index = FR_N ;
-                }
-            }else if( ptr[0] == 'S' || ptr[0] == 's' )
-            {
-                if( ptr[1] == 'W' || ptr[1] == 'w' )
-                    item->index = FR_SW ;
-                else if( ptr[1] == 'E' || ptr[1] == 'e' )
-                    item->index = FR_SE ;
-                else
-                {
-                    register int i = 0;
-                    while( i < 5 && ptr[i] != '\0' ) ++i;
-                    if( ptr[i] == 'W' || ptr[i] == 'w' )
-                        item->index = FR_SW ;
-                    else if( ptr[i] == 'E' || ptr[i] == 'e' )
-                        item->index = FR_SE ;
-                    else
-                        item->index = FR_S ;
-                }
-            }
-            ++pos;
+            item->index = direction2index(stored->argv[pos++]);
         }
 		switch (item->type)
 		{
@@ -1117,8 +1151,8 @@ ReadFlagItem (unsigned long *set_flags, unsigned long *flags, FreeStorageElem *
 			stored->sub != NULL )
 			return 0;
 
-		if (stored->argc > 1)
-			value = (atol (stored->argv[1]) > 0);
+		if (stored->argc >= 1)
+			value = (atol (stored->argv[0]) > 0);
 
 		while (xref->flag != 0)
 		{
