@@ -254,8 +254,9 @@ grab_root_asimage( ScreenInfo *scr, Window target, Bool screenshot )
 	{
 		Window wdumm ;
     	unsigned int udumm ;
-		int bw = 0 ;
-		int tx, ty, tw, th ;
+		unsigned int bw = 0 ;
+		int tx, ty ;
+		unsigned int tw, th ;
 
 	    if( XGetGeometry(dpy, target, &wdumm, &tx, &ty, &tw, &th, &bw, &udumm )!= 0 )
 		{
@@ -514,19 +515,34 @@ mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int heig
 	 case TEXTURE_SCALED_PIXMAP:
 	 	if( get_flags( style->set_flags, F_SLICE ) )
 		{
-			im = slice_asimage (ASDefaultVisual, style->back_icon.image, 
+			im = slice_asimage2(ASDefaultVisual, style->back_icon.image, 
 								style->slice_x_start, style->slice_x_end,
 								style->slice_y_start, style->slice_y_end,
-								preflip_width, preflip_height, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);			   
+								preflip_width, preflip_height, True, 
+								ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);			   
 		}else	 
-			im = scale_asimage (ASDefaultVisual, style->back_icon.image, preflip_width, preflip_height, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
+			im = scale_asimage (ASDefaultVisual, style->back_icon.image, 
+								preflip_width, preflip_height, 
+								ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
 		if( flip != 0 )
 			im = mystyle_flip_image( im, width, height, flip );
 		break;
 	 case TEXTURE_SHAPED_PIXMAP:
 	 case TEXTURE_PIXMAP:
-		 im = tile_asimage (ASDefaultVisual, style->back_icon.image,
-							0, 0, preflip_width, preflip_height, TINT_LEAVE_SAME, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
+	 	 if( get_flags( style->set_flags, F_SLICE ) )
+		 {
+		 	im = slice_asimage2(ASDefaultVisual, style->back_icon.image, 
+								style->slice_x_start, style->slice_x_end,
+								style->slice_y_start, style->slice_y_end,
+								preflip_width, preflip_height, False, 
+								ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);			   
+		 }else
+		 {	
+		 	im = tile_asimage ( ASDefaultVisual, style->back_icon.image,
+								0, 0, 
+								preflip_width, preflip_height, TINT_LEAVE_SAME, 
+								ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
+		 }
 		 LOCAL_DEBUG_OUT( "back_icon.image = %p,im = %p, preflip_size=%dx%d", style->back_icon.image, im, preflip_width, preflip_height );
 		 if( flip != 0 )
 		 	im = mystyle_flip_image( im, width, height, flip );
@@ -539,12 +555,15 @@ mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int heig
 		{
 			if (style->texture_type == TEXTURE_TRANSPARENT || style->texture_type == TEXTURE_TRANSPARENT_TWOWAY)
 			{
-                im = tile_asimage (ASDefaultVisual, ASDefaultScr->RootImage, root_x-ASDefaultScr->RootClipArea.x, root_y-ASDefaultScr->RootClipArea.y,
-									width, height, style->tint, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
+                im = tile_asimage ( ASDefaultVisual, ASDefaultScr->RootImage, 
+									root_x-ASDefaultScr->RootClipArea.x, root_y-ASDefaultScr->RootClipArea.y,
+									width, height, style->tint, 
+									ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
 			} else
 			{
 				 ASImageLayer  layers[2];
 				 ASImage      *scaled_im = NULL;
+				 Bool do_scale = False ;
 				 int           index = 1;      /* default is alphablending !!! */
 
 				 init_image_layers (&layers[0], 2);
@@ -570,27 +589,35 @@ mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int heig
 				 layers[0].clip_height = height;
 
 				 layers[1].im = im?im:style->back_icon.image;
-				 if (style->texture_type >= TEXTURE_SCALED_TRANSPIXMAP &&
-					 style->texture_type < TEXTURE_SCALED_TRANSPIXMAP_END)
-				 {
-				 	if( get_flags( style->set_flags, F_SLICE ) )
-					{
-						scaled_im = slice_asimage (ASDefaultVisual, layers[1].im, 
+				 	
+				 do_scale = (style->texture_type >= TEXTURE_SCALED_TRANSPIXMAP &&
+					 		 style->texture_type < TEXTURE_SCALED_TRANSPIXMAP_END);
+
+				if( get_flags( style->set_flags, F_SLICE ) )
+				{
+					scaled_im = slice_asimage2 (ASDefaultVisual, layers[1].im, 
 												style->slice_x_start, style->slice_x_end,
 												style->slice_y_start, style->slice_y_end,
-												preflip_width, preflip_height, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);			   
-					}else	 
-						 scaled_im = scale_asimage (ASDefaultVisual, layers[1].im, preflip_width, preflip_height,
-													ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
+												preflip_width, preflip_height, do_scale, 
+												ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);			   
+				}else if( do_scale )  
+				{
+					scaled_im = scale_asimage ( ASDefaultVisual, layers[1].im, 
+												preflip_width, preflip_height,
+											    ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
+ 				}
+				 if (scaled_im)	 
+				 {	
 					 /* here layers[1].im is always style->back_icon.image, so we should not destroy it ! */
-					 if (scaled_im)
-						 layers[1].im = mystyle_flip_image( scaled_im, width, height, flip );
-						 /* scaled_im got destroyed in mystyle_flip_image if it had to be */
-				 }
-				 if( flip != 0 && layers[1].im == style->back_icon.image )
+					 layers[1].im = mystyle_flip_image( scaled_im, width, height, flip );
+					 /* scaled_im got destroyed in mystyle_flip_image if it had to be */
+				 }else if( flip != 0 && layers[1].im == style->back_icon.image )
 				 {
 					/* safely assuming that im is NULL at this point,( see logic above ) */
-					 layers[1].im = im = flip_asimage( ASDefaultVisual, layers[1].im, 0, 0, width, height, flip, ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT );
+					 layers[1].im = im = flip_asimage( ASDefaultVisual, layers[1].im, 
+					 								   0, 0, 
+													   width, height, flip, 
+													   ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT );
 				 }
 				 layers[1].merge_scanlines = layers[0].merge_scanlines;
 				 layers[1].dst_x = 0;
@@ -601,7 +628,9 @@ mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int heig
 				 layers[1].clip_height = height;
 
 				{
-					ASImage *tmp = merge_layers (ASDefaultVisual, &layers[0], 2, width, height, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
+					ASImage *tmp = merge_layers(ASDefaultVisual, &layers[0], 2, 
+												width, height, 
+												ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
 					if( tmp )
 					{
 						if( im )
@@ -1186,7 +1215,8 @@ mystyle_inherit_font (MyStyle * style, MyFont * font)
 ASImageBevel *
 mystyle_make_bevel (MyStyle * style, ASImageBevel * bevel, int hilite, Bool reverse)
 {
-	if (style && hilite != 0)
+	if (style && (hilite&HILITE_MASK) != 0 && 
+		(hilite&(NO_HILITE_INLINE|NO_HILITE_OUTLINE)) != (NO_HILITE_INLINE|NO_HILITE_OUTLINE))
 	{
         int extra_hilite = get_flags (hilite, EXTRA_HILITE)?2:0;
 		if (bevel == NULL)
@@ -1221,8 +1251,7 @@ mystyle_make_bevel (MyStyle * style, ASImageBevel * bevel, int hilite, Bool reve
                 bevel->left_inline = bevel->top_inline = bevel->right_inline = bevel->bottom_inline = extra_hilite;
 #endif
             }
-        }else if( get_flags( hilite, NO_HILITE_INLINE ) )
-			clear_flags( hilite, NO_HILITE_INLINE );
+        }
 
 #endif
         if (get_flags (hilite, LEFT_HILITE))
