@@ -20,7 +20,9 @@
 #include "../../configure.h"
 #define LOCAL_DEBUG
 
+
 #include "asinternals.h"
+#include "../../libAfterConf/afterconf.h"
 
 #include <signal.h>
 #include <fcntl.h>
@@ -40,8 +42,6 @@
 ASFlagType AfterStepState = 0; /* default status */
 
 /* Config : */
-struct ASDatabase    *Database = NULL;
-
 ASVector     *Modules       = NULL;
 int           Module_fd     = 0;
 int           Module_npipes = 8;
@@ -173,10 +173,6 @@ SHOW_CHECKPOINT;
 SHOW_CHECKPOINT;
 	XSync (dpy, 0);
 SHOW_CHECKPOINT;
-#ifdef XSHMIMAGE
-	enable_shmem_images ();
-SHOW_CHECKPOINT;
-#endif
     set_parent_hints_func( afterstep_parent_hints_func ); /* callback for collect_hints() */
 SHOW_CHECKPOINT;
     SetupModules();
@@ -259,7 +255,6 @@ SHOW_CHECKPOINT;
 
     /* Load config ... */
     /* read config file, set up menus, colors, fonts */
-    InitDatabase (False);
     LoadASConfig (0, PARSE_EVERYTHING);
 
     /* Reparent all the windows and setup pan frames : */
@@ -349,7 +344,7 @@ CreateManagementWindows()
 	LOCAL_DEBUG_OUT( "Scr.SizeWindow = %lX;", Scr.SizeWindow );
     /* create a window which will accept the keyboard focus when no other
 	   windows have it */
-    attr.event_mask = KeyPressMask | FocusChangeMask;
+    attr.event_mask = KeyPressMask | FocusChangeMask | AS_ROOT_EVENT_MASK;
     attr.override_redirect = True;
     Scr.ServiceWin = create_visual_window (Scr.asv, Scr.Root, 0, 0, 1, 1, 0,
                                            InputOutput, CWEventMask | CWOverrideRedirect,
@@ -568,7 +563,7 @@ CaptureAllWindows (ScreenInfo *scr)
             if( (wm_state == IconicState) || (attr.map_state != IsUnmapped))
 			{
 				LOCAL_DEBUG_OUT( "adding window %lX", children[i] );	  
-                AddWindow( children[i] );
+                AddWindow( children[i], False );
 			}
 		}
 
@@ -649,6 +644,11 @@ Done (Bool restart, char *command )
 
 LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?command:"");
 
+	XSelectInput( dpy, Scr.Root, 0 );	   
+	SendPacket (-1, M_SHUTDOWN, 0);
+	FlushAllQueues(); 
+	sleep_a_millisec(1000);
+
     /* lets duplicate the string so we don't accidental;y delete it while closing self down */
     if( restart )
 	{
@@ -708,7 +708,6 @@ LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?
 		/* freeing up memory */
     	destroy_assession( Session );
 		destroy_asenvironment( &Environment );
-    	InitDatabase(True);
     	free_func_hash ();
     	/* pixmap references */
     	build_xpm_colormap (NULL);
