@@ -41,14 +41,26 @@
 #include <sys/types.h>
 #include <sys/un.h>							   /* for struct sockaddr_un */
 
+#if TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #include <X11/keysym.h>
 
+#include "../../libAfterStep/desktop_category.h"
 #include "../../libAfterStep/module.h"
 #include "../../libAfterStep/wmprops.h"
 
 static DECL_VECTOR(send_data_type, module_output_buffer);
 
-static void       DeleteQueueBuff (module_t *module);
+static void        DeleteQueueBuff (module_t *module);
 static void       AddToQueue      (module_t *module, send_data_type *ptr, int size, int done);
 
 int           module_listen (const char *socket_name);
@@ -168,11 +180,11 @@ ReadModuleInput (module_t *module, size_t * offset, size_t size, void *ptr)
 static void
 CheckCmdTextSize (module_t *module, CARD32 * size, CARD32 * curr_len, char **text)
 {
-	/* max command length is 255 */
-	if (*size > 255)
+	/* max command length is 1024 */
+	if (*size > 1024)
 	{
         show_error("command from module '%s' is too big (%d)", module->name, *size);
-		*size = 255;
+		*size = 1024;
 	}
 
 	/* need to be able to read in command */
@@ -242,14 +254,13 @@ LOCAL_DEBUG_OUT("Incoming message in proto 1%s","");
 		{
             /* for module->afterstep communications - 32 bit values are always used : */
             CARD32        curr_len;
-            CARD32        tmp32, tmp32_val[2], tmp32_unit[2] ;
+            CARD32        tmp32, tmp32_val[2] = {0,0}, tmp32_unit[2] = {0,0} ;
 			register FunctionData *pfunc = ibuf->func;
-            int i ;
 
 LOCAL_DEBUG_OUT("Incoming message in proto 2%s","");
             if (pfunc == NULL)
 			{
-				pfunc = (FunctionData *) safemalloc (sizeof (FunctionData));
+				pfunc = (FunctionData *) safecalloc (1, sizeof (FunctionData));
 				init_func_data (pfunc);
 				ibuf->func = pfunc;
 			}
@@ -287,15 +298,24 @@ LOCAL_DEBUG_OUT("Incoming message in proto 2%s","");
 			}
             LOCAL_DEBUG_OUT( "text_size = %ld, pfunc->text = %p", ibuf->text_size, pfunc->text );
             if (res > 0)
+			{	
                 res = ReadModuleInput (module, &offset, sizeof (tmp32_val), &(tmp32_val[0]));
+				if (res > 0)
+				{	
+    	        	pfunc->func_val[0] = tmp32_val[0] ;
+					pfunc->func_val[1] = tmp32_val[1] ;
+				}
+			}
+
 
 			if (res > 0)
+			{	
                 res = ReadModuleInput (module, &offset, sizeof (tmp32_unit), &(tmp32_unit[0]));
-
-            for( i = 0 ; i < MAX_FUNC_ARGS ; ++i )
-            {
-                pfunc->func_val[i] = tmp32_val[i] ;
-                pfunc->unit_val[i] = tmp32_unit[i] ;
+				if (res > 0)
+				{	
+					pfunc->unit_val[0] = tmp32_unit[0] ;
+					pfunc->unit_val[1] = tmp32_unit[1] ;
+				}
             }
 
 			if (res > 0 && IsValidFunc (pfunc->func))
@@ -464,24 +484,28 @@ PositiveWrite (unsigned int channel, send_data_type *ptr, int size)
 		return -1;
 
 	AddToQueue (module, ptr, size, 0);
-    if (get_flags(module->mask, M_LOCKONSEND) && !is_server_grabbed() )
+    if (get_flags(module->lock_on_send_mask, mask) && !is_server_grabbed() )
 	{
         int           res ;
 		int wait_count = 0 ; 
         do
         {
             if( (res = FlushQueue (module)) >= 0 )
+			{
+				sleep_a_millisec(10);/* give it some time to react */	  
                 res = HandleModuleInput (module);
-            if ( res > 0  ) /* need to run command */
+            }
+			if ( res > 0  ) /* need to run command */
             {
+				LOCAL_DEBUG_OUT( "replay received while waiting for UNLOCK, func = %ld, F_UNLOCK = %d", module->ibuf.func->func, F_UNLOCK );
                 if( module->ibuf.func->func == F_UNLOCK )
                     return size;
                 RunCommand (module->ibuf.func, channel, module->ibuf.window);
             }
-			sleep_a_millisec(100);/* give it some time to react */
+			sleep_a_millisec(10);/* give it some time to react */
 			++wait_count ;
 			/* module has no more then 20 seconds to unlock us */
-        }while( res >= 0 && wait_count < 200 );
+        }while( res >= 0 && wait_count < 2000 );
 	}
 	return size;
 }
@@ -684,26 +708,69 @@ DeadPipe (int nonsense)
 }
 
 
+int FindModuleByName (char *name)
+{
+	int module = -1;
+	if( Modules ) 
+	{	
+    	wild_reg_exp  *wrexp = compile_wild_reg_exp( name );
+
+    	if ( wrexp != NULL )
+    	{
+        	register int i = MODULES_NUM;
+        	register module_t *list = MODULES_LIST ;
+
+        	while( --i >= 0 )
+            	if (list[i].fd > 0)
+            	{
+                	LOCAL_DEBUG_OUT( "checking if module %d \"%s\" matches regexp \"%s\"", i, list[i].name, name);
+                	if (match_wild_reg_exp( list[i].name, wrexp ) == 0 )
+					{	
+                    	module = i ; 
+						break;
+					}
+            	}
+        	destroy_wild_reg_exp( wrexp );
+		}
+    }
+	return module;
+}
+
+char *
+GetModuleCmdLineByName(char *name)
+{
+	int module = FindModuleByName (name);
+	if( module >= 0 ) 
+	{	
+		register module_t *list = MODULES_LIST ;	
+		if( list[module].cmd_line ) 
+			return mystrdup( list[module].cmd_line ) ;
+		else
+			return mystrdup( list[module].name );
+	}
+	return NULL ;
+}
+
 void
 KillModuleByName (char *name)
 {
-    wild_reg_exp  *wrexp = compile_wild_reg_exp( name );
+	int module = FindModuleByName (name);
+	if( module >= 0 ) 
+	{	
+		register module_t *list = MODULES_LIST ;	
+        KillModule (&(list[module]), False);
+	}
+}
 
-    if ( wrexp != NULL && Modules)
-    {
-        register int i = MODULES_NUM;
-        register module_t *list = MODULES_LIST ;
-
-        while( --i >= 0 )
-            if (list[i].fd > 0)
-            {
-                LOCAL_DEBUG_OUT( "checking to kill module %d \"%s\", regexp \"%s\"", i, list[i].name, name);
-                if (match_wild_reg_exp( list[i].name, wrexp ) == 0 )
-                    KillModule (&(list[i]), False);
-            }
-        destroy_wild_reg_exp( wrexp );
-    }
-	return;
+void
+KillAllModulesByName (char *name)
+{
+	int module ;
+	while( (module = FindModuleByName (name)) >= 0 )
+	{	
+		register module_t *list = MODULES_LIST ;	
+        KillModule (&(list[module]), False);
+	}
 }
 
 void
@@ -867,12 +934,16 @@ RunCommand (FunctionData * fdata, unsigned int channel, Window w)
 	{
 	 case F_SET_MASK:
          module->mask = fdata->func_val[0];
+		 module->lock_on_send_mask = fdata->func_val[1];
 		 break;
 	 case F_SET_NAME:
-        if (module->name != NULL)
-            free (module->name);
-        module->name = fdata->name;
+        set_string( &(module->name), fdata->name );
         fdata->name = NULL;
+		if( fdata->text ) 
+		{	
+	        set_string( &(module->cmd_line), fdata->text );
+    	    fdata->text = NULL;
+		}
         break;
      case F_UNLOCK:
 		 toret = 66;
@@ -914,6 +985,7 @@ RunCommand (FunctionData * fdata, unsigned int channel, Window w)
 	 default:
         {
             ASEvent event;
+			Bool defered = False ;
             memset( &event, 0x00, sizeof(ASEvent) );
             event.w = w;
             if ((event.client = window2ASWindow(w)) == NULL )
@@ -926,16 +998,19 @@ RunCommand (FunctionData * fdata, unsigned int channel, Window w)
                 event.x.xbutton.x_root = event.client->frame_canvas->root_x+1;
                 event.x.xbutton.y_root = event.client->frame_canvas->root_y+1;
             }
+			
             event.x.xany.type = ButtonRelease;
             event.x.xbutton.button = 1;
             event.x.xbutton.x = 0;
             event.x.xbutton.y = 0;
             event.x.xbutton.subwindow = None;
             event.context = C_FRAME;
-            event.scr = &Scr ;
+            event.scr = ASDefaultScr ;
             event.event_time = Scr.last_Timestamp ;
             /* there must be no deffering on module commands ! */
-            ExecuteFunctionExt (fdata, &event, channel, True);
+
+			defered = ( w != None || !IsWindowFunc (fdata->func) );
+            ExecuteFunctionExt (fdata, &event, channel, defered);
         }
     }
     free_func_data (fdata);
@@ -963,7 +1038,7 @@ broadcast_window_name( ASWindow *asw )
     if( asw )
 	{	
         SendString( -1, M_WINDOW_NAME, asw->w, asw->frame,
-                    asw, ASWIN_NAME(asw), asw->hints->names_encoding[0]);
+                    asw, ASWIN_NAME(asw), get_hint_name_encoding(asw->hints, 0));
         SendString( -1, M_WINDOW_NAME_MATCHED, asw->w, asw->frame,
                     asw, asw->hints->matched_name0, asw->hints->matched_name0_encoding );
 	}
@@ -973,8 +1048,10 @@ void
 broadcast_icon_name( ASWindow *asw )
 {
     if( asw )
+	{
         SendString( -1, M_ICON_NAME, asw->w, asw->frame,
-                    asw, ASWIN_ICON_NAME(asw), asw->hints->names_encoding[asw->hints->icon_name_idx]);
+                    asw, ASWIN_ICON_NAME(asw), get_hint_name_encoding(asw->hints, asw->hints->icon_name_idx));
+	}
 }
 
 void
@@ -983,9 +1060,9 @@ broadcast_res_names( ASWindow *asw )
     if( asw )
     {
         SendString( -1, M_RES_CLASS, asw->w, asw->frame,
-                    asw, asw->hints->res_class, asw->hints->names_encoding[asw->hints->res_class_idx]);
+                    asw, asw->hints->res_class, get_hint_name_encoding(asw->hints, asw->hints->res_class_idx));
         SendString( -1, M_RES_NAME, asw->w, asw->frame,
-                    asw, asw->hints->res_name, asw->hints->names_encoding[asw->hints->res_name_idx]);
+                    asw, asw->hints->res_name, get_hint_name_encoding(asw->hints, asw->hints->res_name_idx));
     }
 }
 
@@ -1009,3 +1086,83 @@ broadcast_config (send_data_type event_type, ASWindow * t)
     SendConfig( -1, event_type, t );
 }
 
+/********************************************************************************/
+/* module list menus regeneration :                                             */
+/********************************************************************************/
+
+static inline void
+module_t2func_data( FunctionCode func, module_t *module, FunctionData *fdata, char *scut ) 
+{
+	fdata->func = func;
+    fdata->name = mystrdup(module->name);
+    fdata->text = mystrdup(module->name);
+	if (++(*scut) == ('9' + 1))
+		(*scut) = 'A';		/* Next shortcut key */
+    fdata->hotkey = (*scut);
+}	   
+
+MenuData *
+make_module_menu( FunctionCode func, const char *title, int sort_order )
+{
+    MenuData *md ;
+    MenuDataItem *mdi ;
+    FunctionData  fdata ;
+    char          scut = '0';                  /* Current short cut key */
+	module_t *modules ;
+	int i, max_i ;
+
+	if ( Modules == NULL ) 
+        return NULL;
+    
+	if( (md = create_menu_data ("@#%module_menu%#@")) == NULL )
+        return NULL;
+
+	modules = MODULES_LIST ;
+	max_i = MODULES_NUM ;
+
+    memset(&fdata, 0x00, sizeof(FunctionData));
+    fdata.func = F_TITLE ;
+    fdata.name = mystrdup(title);
+    add_menu_fdata_item( md, &fdata, NULL, NULL );
+
+	if( sort_order == ASO_Alpha ) 
+	{
+		FunctionData **menuitems = safecalloc(sizeof(FunctionData *), max_i);
+        
+		for( i = 0 ; i < max_i ; ++i) 
+		{
+			menuitems[i] = safecalloc(1, sizeof(FunctionData));
+			module_t2func_data( func, &(modules[i]), menuitems[i], &scut ); 
+        }
+		qsort(menuitems, i, sizeof(FunctionData *), compare_func_data_name);
+		for( i = 0 ; i < max_i ; ++i) 
+		{
+			ASDesktopEntry *de ;
+			char *minipixmap = NULL ; 
+			de = fetch_desktop_entry( AfterStepCategories, menuitems[i]->name );
+			if( de && de->fulliconname ) 
+				minipixmap = de->fulliconname ;
+
+			if( (mdi = add_menu_fdata_item( md, menuitems[i], minipixmap, NULL)) != NULL ) 
+				set_flags( mdi->flags, MD_ScaleMinipixmapDown );
+			safefree( menuitems[i] ); /* scrubba-dub-dub */
+		}
+		safefree(menuitems);
+    }else /* if( sort_order == ASO_Circulation || sort_order == ASO_Stacking ) */
+	{
+		for( i = 0 ; i < max_i ; ++i) 
+        {
+			ASDesktopEntry *de ;
+			char *minipixmap = NULL ; 
+            module_t2func_data( func, &(modules[i]), &fdata, &scut ); 
+			de = fetch_desktop_entry( AfterStepCategories, fdata.name );
+			if( de && de->fulliconname ) 
+				minipixmap = de->fulliconname ;
+            if( (mdi = add_menu_fdata_item( md, &fdata, minipixmap, NULL )) != NULL ) 
+				set_flags( mdi->flags, MD_ScaleMinipixmapDown );
+        }
+    }
+    return md;
+}
+
+
